#include <alloca.h>
#include <cstdio>
#include <cstdlib>

#include "Domain.h"

#include "BinFileHandler.h"
#include "EnsightFileHandler.h"

Point3D p0;

//-----------------------------------------------------------------

Domain::Domain(int numClus) : elements(0), node(p0) 
{

  decomp = new Decomp3D();
  numClusters = numClus;
  numNode = 0;
  numElem = 0;


  // PJSA: moved bcdata to Domain class so it can be used for fluid as well as structure
  numIDis6 = 0;
  numIDis = 0;
  numIVel = 0;
  numDirichlet = 0;
  numNeuman = 0;
  //numSommer = 0;
  //numLMPC = 0;
  numConvBC = 0;
  numITemp = 0;
  //numComplexDirichlet = 0;
  //numComplexNeuman = 0;
                                                                                                             
                                                                                                             
  // set bc pointers to zero
  dbc = 0;
  nbc = 0;
  cvbc = 0;
  iDis = 0;
  iDis6 = 0;
  iVel = 0;
  iTemp = 0;
                                                                                                             
                                                                                                             
  // set per cluster members to zero
  dbcPerClus = 0;
  nbcPerClus = 0;
  cvbcPerClus = 0;
  iDisPerClus = 0;
  iDis6PerClus = 0;
  iVelPerClus = 0;
  iTempPerClus = 0;

}

//---------------------------------------------------------------------

void Domain::getDecomp(const char *name, int &numNumCpus, int *numCpus) 
{ 

  decomp->getDecomp(name, numElem);

  int numSubs = decomp->getNumSub();

  if (numNumCpus == 0) {
    numNumCpus = 1;
    numCpus[0] = numSubs;
  }

  fprintf(stdout, "... Number of cpu(s) = %d\n", numCpus[0]);
  fprintf(stdout, "... Number of cluster(s) = %d\n", numClusters);
  fprintf(stdout, "... Number of subdomain(s) = %d\n", numSubs);

  for (int i=0; i<numNumCpus; ++i) {
    if (numCpus[i] > numSubs) {
      numCpus[i] = numSubs;
      fprintf(stderr, " *** WARNING: number of cpus set to %d\n", numCpus[i]);
    }
  }

  if (numClusters > numSubs) {
    numClusters = numSubs;
    fprintf(stderr, " *** WARNING: number of clusters set to %d\n", numClusters);
  }

}

//-----------------------------------------------------------------------

void Domain::makeConnectivity(Connectivity *elemToClus) 
{

  decomp->makeConnectivity(numClusters, numElem, numNode, elements+0, elemToClus);
  decomp->getDomainConnect();

}

//---------------------------------------------------------------------
//TODO do the same for the match pts

void Domain::createClusterToGlobalNodeMap()  
{

  Connectivity *nodeToSub = decomp->getNodeToSub();

  // sort the node to sub connectivity
  int iNode;
  for(iNode = 0; iNode < nodeToSub->csize(); ++iNode)
    std::sort((*nodeToSub)[iNode], (*nodeToSub)[iNode]+nodeToSub->num(iNode));

  // order the nodes in each cluster according to
  // the number of subdomains they are connected to

  clusterNodeMap = new int *[numClusters];
  
  Connectivity *clusToNode = decomp->getClusToNode();
  for (int iClus = 0; iClus < numClusters; iClus++)  {

    //int numNodes = nodeToSub->csize();
    int numNodes = clusToNode->num(iClus);

    // create a pairing of unique id and node number
    std::pair<int,int> *nodeMap = new std::pair<int,int>[numNodes];

    // order the nodes and return the actual number of nodes
    numNodes = decomp->remapNodes(nodeMap, iClus);

    // populate clusterNodeMap
    clusterNodeMap[iClus] = new int[numNodes];
    for (int iNode = 0; iNode < numNodes; iNode++)
      clusterNodeMap[iClus][iNode] = nodeMap[iNode].second;
    
  }

}

//---------------------------------------------------------------------

void Domain::renumberNodes()  
{

  Connectivity *clusToNode = decomp->getClusToNode();
  Connectivity *clusToSub = decomp->getClusToSub();
  Connectivity *subToElem = decomp->getSubToElem();

  // create global to local cluster node numbering map

  gl2ClNodeMap = new MapVec[numClusters];

  int iClus;
  for (iClus = 0; iClus < numClusters; iClus++)
    for (int iNode = 0; iNode < clusToNode->num(iClus); iNode++)
      gl2ClNodeMap[iClus][ clusterNodeMap[iClus][iNode] ] = iNode;

  // renumber element nodes

  for (iClus = 0; iClus < numClusters; iClus++) {

    for (int iSub = 0; iSub < clusToSub->num(iClus); iSub++) {
      
      int glSub = (*clusToSub)[iClus][iSub];
      
      for (int iElem = 0; iElem < subToElem->num(glSub); iElem++) {
        int elemI = (*subToElem)[glSub][iElem];
        elements[elemI]->renumberNodes(gl2ClNodeMap[iClus]);
      }
      
    }

  }

}

//---------------------------------------------------------------------

void Domain::printGlobalConnectivity(const char *prefix)
{

  Connectivity *clusToSub = decomp->getClusToSub();
  Connectivity *subToSub = decomp->getSubToSub();
  
  char name[MAXLINE];
  sprintf(name, "%s.con", prefix);
  BinFileHandler file(name, "wb", FILE_VER);

  fprintf(stdout, "... Writing \'%s\'\n", name);

  clusToSub->write(file);
  subToSub->write(file);

}

//---------------------------------------------------------------------

void Domain::printCpuToSubMap(int numCpus, const char *prefix, int type) 
{

  decomp->writeCpuToSubMap(numCpus, prefix, type); 

}

//----------------------------------------------------------------

void Domain::printNodes(int iClus, BinFileHandler &outFileGeo, Point3D *nds)
{

  Connectivity *clusToNode = decomp->getClusToNode();
  int numNodes = clusToNode->num(iClus);

  for (int iNode = 0; iNode < numNodes; iNode++) {
    // output global node number
    outFileGeo.write( clusterNodeMap[iClus]+iNode, 1);

    // write out node coordinates
    outFileGeo.write(nds[ clusterNodeMap[iClus][iNode] ].getPoint(), 3);
  }

}

//----------------------------------------------------------------

void Domain::printNodes(int iClus, EnsightFileHandler &outFileGeo, Point3D *nds)
{

  Connectivity *clusToNode = decomp->getClusToNode();
  int numNodes  = clusToNode->num(iClus);
  int   *nodeid = new int[numNodes];
  float *xcoord = new float[numNodes];
  float *ycoord = new float[numNodes];
  float *zcoord = new float[numNodes];

  // Fill x,y,z node coordinates tables
  for (int iNode = 0; iNode < numNodes; iNode++) {
    double *xyz;

    xyz = nds[ clusterNodeMap[iClus][iNode] ].getPoint();

    nodeid[iNode] =  clusterNodeMap[iClus][iNode];
    xcoord[iNode] =  xyz[0];
    ycoord[iNode] =  xyz[1];
    zcoord[iNode] =  xyz[2];
  }

  outFileGeo.write("coordinates");
  outFileGeo.write( &numNodes, 1);
  outFileGeo.write( nodeid, numNodes, 1);
  outFileGeo.write( xcoord, numNodes, 1);
  outFileGeo.write( ycoord, numNodes, 1);
  outFileGeo.write( zcoord, numNodes, 1);
 
  delete [] nodeid;
  delete [] xcoord;
  delete [] ycoord;
  delete [] zcoord;
}

//---------------------------------------------------------------------

void Domain::printRangesInfo(BinFileHandler &file, int glSub,
			     Connectivity *subToObj, MapVec &gl2Cl) 
{ 

  int numObj = subToObj->num(glSub);

  int *locObj = new int[numObj];

  for (int i = 0; i < numObj; ++i)
    locObj[i] = gl2Cl[ (*subToObj)[glSub][i] ];

  std::stable_sort(locObj, locObj + numObj);

  int numRanges = 0;
  int numSingles = 0;

  if (numObj > 1) {

    int i;
    for (i = 0; i < numObj-1; i++) {
      if ( locObj[i]+1 == locObj[i+1] ) {
	numRanges++;
	i++;
	while ( (i+1) < numObj  && locObj[i+1] == locObj[i] + 1 )
	  i++;
      }
      else
	numSingles++;
    }

    if (locObj[numObj-1] != locObj[numObj-2]+1)
      numSingles++;

    int (*ranges)[2] = new int[numRanges][2];
    int *singles = new int[numSingles];

    int iRange = 0;
    int iSingle = 0;

    for (i = 0; (i+1) < numObj; i++)  {
      if ( locObj[i]+1 == locObj[i+1] )  {
	ranges[iRange][0] = locObj[i];
	i++;
	while ( (i+1) < numObj  && locObj[i+1] == locObj[i] + 1 )
	  i++;
	ranges[iRange][1] = locObj[i];
	iRange++;
      }
      else
	singles[iSingle++] = locObj[i];
    }

    if (locObj[numObj-1] != locObj[numObj-2]+1)
      singles[iSingle] = locObj[numObj-1];
    else
      ranges[iRange-1][1] = locObj[numObj-1];

    file.write(&numRanges, 1);
    file.write(&numSingles, 1);
    file.write(reinterpret_cast<int *>(ranges), 2*numRanges);
    file.write(singles, numSingles);

    if (ranges) delete [] ranges;
    if (singles) delete [] singles;

  }
  else if (numObj == 1) {
    file.write(&numRanges, 1);
    numSingles++;
    file.write(&numSingles, 1);
    file.write(&locObj[0], 1);
  }
  else {
    file.write(&numRanges, 1);
    file.write(&numSingles, 1);
  }

  if (locObj) delete [] locObj;

}

//------------------------------------------------------------

void Domain::printInterfaceInfo(BinFileHandler &file, int numSubNeighbors, 
				int *subNeighbors, Connectivity *interfNodes,
				MapVec &gl2ClNode)  
{

  file.write(&numSubNeighbors, 1);
  if (numSubNeighbors > 0) {
    file.write(subNeighbors, numSubNeighbors);
    interfNodes->renumberTargets(gl2ClNode);
    interfNodes->write(file);
  }
}

//------------------------------------------------------------
